# ToBybuR v2.1, un bot qui fait plein de choses sur les bouchots
# Copyright (c) 2006-2009 Nicolas Sergent
#  
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
#
# Several lines of this program (reading the config files and posting to
# the tribune) are inspired by rbcoincoin, written by Fabien Penso
# (http://rubyforge.org/projects/rbcoincoin/), and released under GPL ("either
# version 2, or (at your option) any later version").
#
# Some others (interacting with the sql server) are inspired by the fossileBouchot
# (http://zorel.org/static/fossileBouchot/) by Aurélien DEHAY.
# 


require 'net/http'
require 'net/https'
require 'rexml/document'
require 'rexml/encoding'
require 'time'
require 'dbi'

require './ToBylib.rb'

include REXML

Thread.current["name"]="global"

config_file = "ToBybuR.xml"
log_file = "ToBylogs.txt"

if not File.exists?(config_file)
  puts "Le fichier de configuration (" + config_file + ") n'existe pas. ToBybuR ne peut pas fonctionner sans lui..."
  exit
end

##si on nous avait demandé de stopper avant, on efface le fichier stop
if File.exists?("./stop"); File.delete("./stop"); end

##on ouvre le fichier de logs (accessible par tous les modules)
$logfile = File::open(log_file, 'a')

log("On commence!")

##On commence par charger les options, d'abord de ToBybuR, puis des tribunes.
$config = Config.new(config_file)

##on se connecte à la base de donnée
db_connect()

##Et on créé le hash qui contiendra les configs des tribunes
$tribune=Hash.new

$tribune_threads = Hash.new

log("Paré au lancement !")


###############################################################################
#                            ÇA COMMENCE !!!                                  #
###############################################################################
Thread.abort_on_exception = true
#maintenant qu'on a tout ce dont on a besoin, on peut rentrer dans la boucle
loop do

  ##si un fichier stop est créé dans le répertoire, le programme quitte gentillement
  ##(en demandant d'abord aux threads de s'arrêter)
  if File.exists?("./stop")
    log("On nous demande de nous arrêter")
    $tribune_threads.each{|name, thread|
      if thread[:is_waiting]
        log(name.to_s + " dors, on la tue")
        thread.exit
      end
    }
    $tribune_threads.each{|name, thread|
      thread.join
    }
    break
  end

  ##si la config du bot a été modifiée, on la recharge (si le fichier existe encore:)
  if File.exists?(config_file) && File.ctime(config_file) > $config.file_time
    log(config_file + " a été modifié : on le recharge")
    $config.read_conf(config_file)
  end

  ##et on lance un thread pour chaque tribune (sauf si cette tribune est encore en train
  ##de tourner dans son thread)
  $config.tribune_list.each{|tribune_name, tribune_conf|
    if ($tribune_threads[tribune_name].nil? || $tribune_threads[tribune_name].alive? == false)
      $tribune_threads[tribune_name] = Thread.new(tribune_name, tribune_conf) {|name, conf|
        Thread.current["name"]=name
        Thread.current["is_waiting"]=false

        ##si la tribune part en quenouille, on veut que seul ce thread meurre, pas tout
        ##le programme et les autres tribunes.
#        begin


          ##si la tribune n'existe pas, on créé l'objet correspondant
          ##typiquement, ça ne se produit qu'au démarrage et à l'ajout d'une tribune
          ##dans le fichier de conf
          if $tribune[name] == nil
            $tribune[name] = Tribune.new(conf)
            $tribune[name].load_modules()
            $tribune[name].prepare()

          ##si la conf de la tribune a changé, on met à jour
          elsif $tribune[name].conf.to_s != conf.to_s
            log("la configuration de la tribune a changé, on la recharge")
            $tribune[name].reload_conf(conf)
          end


          ##on récupère le remote.xml
          board = get_board($config, $tribune[name])
          ##ça, c'est pour tester sans faire chier le serveur
          #remote_xml = REXML::Document.new(File.read("remote.xml"))

          ##on lit tous les nouveaux posts
          if board == nil
            Thread.current["is_waiting"]=true
            sleep $tribune[name].refresh
            Thread.current.exit
          end
          board.get_elements("board/post[@id>#{$tribune[name].last_post}]").reverse_each{|p|


            ##on créé un objet "Poste" qui contient toutes le informations du post
            $tribune[name].current_post = Poste.new($tribune[name],p)
            
            ##seules les moules non blacklistées peuvent utiliser les modules
            ##dans les cas des modules qui ont une "whitelist", seules les moules autorisées peuvent les utiliser
            ##(pour ces modules, l'attribut "whitelist"=1 dans le fichier de conf)
            ##les anonymes sont représentés par le login "" dans la table blacklist
            blacklistage = dbh("select_one", "SELECT * FROM #{$tribune[name].db_prefix}blacklist WHERE login = ?", $tribune[name].current_post.login)


            ##on charge tous les modules actifs, si la moule est autorisée
            $tribune[name].modules_liste.each{|mod|
              if mod.statut == "on"
                if ( mod.whitelist == "1" && blacklistage != nil && blacklistage[mod.nom].to_s == "-1") || ( mod.whitelist != "1" && (blacklistage == nil || blacklistage[mod.nom].to_s != "1"))
                  begin
                    $tribune[name].current_mod = mod
                    load "modules/" + mod.fichier + ".rb"
                  rescue
                    log(mod.nom + " a eu un problème pour traiter le message:\n" + p.to_s)
                    log("Erreur: #{$!}")
                  end
                end
              end
            }


            ##on change la valeur du dernier post enregistré
            $tribune[name].last_post = $tribune[name].current_post.post_id
          }#fin du parcours des nouveaux posts
    
          ##on attend un peu avant de quitter le thread
          ##(et de recommencer avec un nouveau remote.xml, sauf changement de conf)
          Thread.current["is_waiting"]=true
          sleep $tribune[name].refresh

#        rescue ##le big gros rescue de la mort pour tout le thread
#          log("Oops, une erreur est apparue, je ne sais pas quoi en faire : #{$!}")
#        end
      }#fin du thread
    end
  }#fin de la liste des tribunes

  ##on attend une seconde avant de boucler (cette grande boucle ne fait qu'attendre que
  ##les threads des tribunes aient fini, pour les relancer)
  sleep 2

end

##On se déconnecte de la base
$dbh.disconnect if $dbh

##On ferme le fichier de logs
log("Je me termine gentillement, sans douleur")
$logfile.close
