=begin
== NAME

htpasswd.rb -- interface for apache htpasswd files for basic auth (using system crypt).

Copyright (C) 2003 Dmitry Morozhnikov <dmiceman@mail.ru>

Modifié pour prendre en argument un mot de passe déjà crypté ou md5ifié, et ajout d'une fonction pour créer un mot de passe aléatoire

== EXAMPLE

  require 'htpasswd'
  passwd = HtPasswd.new('/home/dm/htpasswd')
  puts 'Users in password file:'
  passwd.each do |user|
    puts user
  end
  puts 'crypted user password: ' + passwd[user]
  passwd[user] = 'new password'
  passwd.delete user

=end

=begin
== EXCEPTIONS

HtPasswdError -- class of exceptions generated by HtPasswd.

=end
class HtPasswdError < RuntimeError
end

class HtPasswd

  include Enumerable

=begin
== CONSTRUCTOR

--- HtPasswd.new(file = './.htpasswd')
      * file -- password file to work with.

=end
  def initialize(file = './.htpasswd')
    @file = file
    @users = Hash.new
    @stamp = nil
  end

=begin

== METHODS

=end

=begin
--- HtPasswd#each
      iterate over user list stored in password file.
      ((-HtPasswd include module Enumerable, so all methods of this module is also provided.-))

=end
  def each
    update
    @users.keys.each {|u| yield u}
  end

  def update
    return @users if @users and @stamp and @stamp >= File.mtime(@file)
    @users = Hash.new
    File.open(@file, File::CREAT | File::RDONLY, 0644) do |f|
      f.flock File::LOCK_SH
      f.each do |l|
        user, pass = l.split ':'
        @users[user] = pass if user and user != '' and pass and pass != ''
      end
      @stamp = f.mtime
      f.flock File::LOCK_UN
    end or raise HtPasswdError, 'cannot open ' + @file + ' for read'
    @users
  end

  private :update
  


=begin
--- HtPasswd#[](user)
--- HtPasswd#[]=(user, password)
--- HtPasswd#delete(user)
      view encrypted user password, set new user password and delete user from password file.

=end
  def [](user)
    update
    @users[user]
  end
  def []=(user, passwd)
    @users[user] = passwd
    update
    if @users[user]
      File.open(@file, File::CREAT | File::TRUNC | File::WRONLY, 0644) do |f|
        f.flock File::LOCK_EX
        @users.keys.each do |u|
          f.puts u + ':' + @users[u]
        end
        @stamp = f.mtime
        f.flock File::LOCK_UN
      end or raise HtPasswdError, 'cannot open ' + @file + ' for write'
    else
      File.open(@file, File::WRONLY | File::CREAT | File::APPEND, 0644) do |f|
        f.flock File::LOCK_EX
        f.puts user + ':' + passwd
        @stamp = f.mtime
        f.flock File::LOCK_UN
      end or raise HtPasswdError, 'cannot open ' + @file + ' for append'
    end
  end
  def delete(user)
    @users.delete user
    update
    File.open(@file, File::CREAT | File::TRUNC | File::WRONLY, 0644) do |f|
      f.flock File::LOCK_EX
      @users.keys.each do |u|
        f.puts u + ':' + @users[u]
      end
      @stamp = f.mtime
      f.flock File::LOCK_UN
    end or raise HtPasswdError, 'cannot open ' + @file + ' for write'
  end

end

def newpass( len )
  chars = ("a".."z").to_a + ("A".."Z").to_a + ("0".."9").to_a
  newpass = ""
  1.upto(len) { |i| newpass << chars[rand(chars.size-1)] }
  newpass
end
